#
# This has been ported by Hiroshi Saito(hiroshi@winpg.jp)
# Copyright notice of the original as below.
#
#
# * << Haru Free PDF Library 2.0.0 >> -- text_demo.c
# *
# * Copyright(c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# *
# * Permission to use, copy, modify, distribute and sell this software
# * and its documentation for any purpose is hereby granted without fee,
# * provided that the above copyright notice appear in all copies and
# * that both that copyright notice and this permission notice appear
# * in supporting documentation.
# * It is provided "as is" without express or implied warranty.
# *
#

@load "pdf"

# ----
# function
# ----

function tan(x)
{
    return sin(x)/cos(x);
}

function show_stripe_pattern(page, x, y)
{
    iy = 0;
    while(iy < 50) {
     HPDF_Page_SetRGBStroke(page, 0.0, 0.0, 0.5);
     HPDF_Page_SetLineWidth(page, 1);
     HPDF_Page_MoveTo(page, x, y + iy);
     HPDF_Page_LineTo(page, x + HPDF_Page_TextWidth(page, "ABCabc123"),
                    y + iy);
     HPDF_Page_Stroke(page);
     iy += 3;
    }

    HPDF_Page_SetLineWidth(page, 2.5);
}

function show_description(page, font, x, y, text)
{

    fsize = HPDF_Page_GetCurrentFontSize(page);
    HPDF_Page_BeginText(page);
    HPDF_Page_SetFontAndSize(page, font, 9);
    HPDF_Page_SetRGBFill(page, 0, 0, 0);
    HPDF_Page_SetTextRenderingMode(page, HPDF_FILL);
    HPDF_Page_TextOut(page, x, y - 12, text);

    HPDF_Page_SetFontAndSize(page, font, fsize);
    HPDF_Page_EndText(page);

    c = HPDF_Page_GetRGBFill(page);
    split(c,c_SP,",");  # R, G, B array
    HPDF_Page_SetRGBFill(page, c_SP[1], c_SP[2], c_SP[3]);
}

# ----
# main
# ----

BEGIN {
    page_title = "font size and color examples.";
    samp_text = "abcdefgABCDEFG123!#$%&+-@?";
    samp_text2 = "The quick brown fox jumps over the lazy dog.";
    fname = "pdf_text_color.pdf";

    pdf = HPDF_New(NULL, NULL);

    # set compression mode 
    HPDF_SetCompressionMode(pdf, HPDF_COMP_ALL);

    # create default-font 
    font = HPDF_GetFont(pdf, "Helvetica", NULL);

    # add a new page object. 
    page = HPDF_AddPage(pdf);

    # print the lines of the page.
    HPDF_Page_SetLineWidth(page, 1);
    HPDF_Page_Rectangle(page, 50, 50, HPDF_Page_GetWidth(page) - 100,
              HPDF_Page_GetHeight(page) - 110);
    HPDF_Page_Stroke(page);

    # print the title of the page(with positioning center). 
    HPDF_Page_SetFontAndSize(page, font, 24);
    tw = HPDF_Page_TextWidth(page, page_title);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page,(HPDF_Page_GetWidth(page) - tw) / 2,
              HPDF_Page_GetHeight(page) - 50, page_title);
    HPDF_Page_EndText(page);

    HPDF_Page_BeginText(page);
    HPDF_Page_MoveTextPos(page, 60, HPDF_Page_GetHeight(page) - 60);

    # * font size
     
    fsize = 6;
    while(fsize < 26) {

        # set style and size of font. 
        HPDF_Page_SetFontAndSize(page, font, fsize);

        # set the position of the text. 
        HPDF_Page_MoveTextPos(page, 0, -5 - fsize);

        # measure the number of characters which included in the page. 
        len = HPDF_Page_MeasureText(page, samp_text,
                     HPDF_Page_GetWidth(page) - 120, HPDF_FALSE, NULL);

        # truncate the text. 
        buf = samp_text;
        HPDF_Page_ShowText(page, buf);

        # print the description. 
        HPDF_Page_MoveTextPos(page, 0, -10);
        HPDF_Page_SetFontAndSize(page, font, 8);
        buf = sprintf("Fontsize=%.0f", fsize);
        HPDF_Page_ShowText(page, buf);

        fsize *= 1.2;
    }

    # * font color
     
    HPDF_Page_SetFontAndSize(page, font, 12);
    HPDF_Page_MoveTextPos(page, 0, -30);
    HPDF_Page_ShowText(page, "Font color");

    HPDF_Page_SetFontAndSize(page, font, 24);
    HPDF_Page_MoveTextPos(page, 0, -20);

    len = length(samp_text);
    split(samp_text, samp_text_SP, "");

    for(i = 0; i < len; i++) {
        r =i /len;
        g = 1 -(i /len);
        buf = samp_text_SP[i];

      HPDF_Page_SetRGBFill(page, r, g, 0.0);
      HPDF_Page_ShowText(page, buf);
    }
    HPDF_Page_MoveTextPos(page, 0, -25);

    split(samp_text, samp_text_SP, "");
    for(i = 0; i < len; i++) {
        r =i /len;
        b = 1 -(i /len);
        buf = samp_text_SP[i];

      HPDF_Page_SetRGBFill(page, r, 0.0, b);
      HPDF_Page_ShowText(page, buf);
    }
    HPDF_Page_MoveTextPos(page, 0, -25);

    split(samp_text, samp_text_SP, "");
    for(i = 0; i < len; i++) {
        b =i /len;
        g = 1 -(i /len);
        buf = samp_text_SP[i];

      HPDF_Page_SetRGBFill(page, 0.0, g, b);
      HPDF_Page_ShowText(page, buf);
    }

    HPDF_Page_EndText(page);

    # * Font rendering mode
     
    HPDF_Page_SetFontAndSize(page, font, 24);
    HPDF_Page_SetRGBFill(page, 0.5, 0.5, 0.0);
    HPDF_Page_SetLineWidth(page, 1.5);

    # PDF_FILL 
    ypos = 380;

    show_description(page, font,  60, ypos,
                "RenderingMode=PDF_FILL");
    HPDF_Page_SetTextRenderingMode(page, HPDF_FILL);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, ypos, "ABCabc123");
    HPDF_Page_EndText(page);

    # PDF_STROKE 
    show_description(page, font, 60, ypos - 45,
                "RenderingMode=PDF_STROKE");
    HPDF_Page_SetTextRenderingMode(page, HPDF_STROKE);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, ypos - 45, "ABCabc123");
    HPDF_Page_EndText(page);

    # PDF_FILL_THEN_STROKE 
    show_description(page, font, 60, ypos - 90,
                "RenderingMode=PDF_FILL_THEN_STROKE");
    HPDF_Page_SetTextRenderingMode(page, HPDF_FILL_THEN_STROKE);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, ypos - 90, "ABCabc123");
    HPDF_Page_EndText(page);

    # PDF_FILL_CLIPPING 
    show_description(page, font, 60, ypos - 135,
                "RenderingMode=PDF_FILL_CLIPPING");
    HPDF_Page_GSave(page);
    HPDF_Page_SetTextRenderingMode(page, HPDF_FILL_CLIPPING);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, ypos - 135, "ABCabc123");
    HPDF_Page_EndText(page);
    show_stripe_pattern(page, 60, ypos - 135);
    HPDF_Page_GRestore(page);

    # PDF_STROKE_CLIPPING 
    show_description(page, font, 60, ypos - 180,
                "RenderingMode=PDF_STROKE_CLIPPING");
    HPDF_Page_GSave(page);
    HPDF_Page_SetTextRenderingMode(page, HPDF_STROKE_CLIPPING);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, ypos - 180, "ABCabc123");
    HPDF_Page_EndText(page);
    show_stripe_pattern(page, 60, ypos - 180);
    HPDF_Page_GRestore(page);

    # PDF_FILL_STROKE_CLIPPING 
    show_description(page, font, 60, ypos - 225,
                "RenderingMode=PDF_FILL_STROKE_CLIPPING");
    HPDF_Page_GSave(page);
    HPDF_Page_SetTextRenderingMode(page, HPDF_FILL_STROKE_CLIPPING);
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, ypos - 225, "ABCabc123");
    HPDF_Page_EndText(page);
    show_stripe_pattern(page, 60, ypos - 225);
    HPDF_Page_GRestore(page);

    # Reset text attributes 
    HPDF_Page_SetTextRenderingMode(page, HPDF_FILL);
    HPDF_Page_SetRGBFill(page, 0, 0, 0);
    HPDF_Page_SetFontAndSize(page, font, 24);

    # * Rotating text
     
    angle1 = 30;                   # A rotation of 30 degrees. 
    rad1 = angle1 / 180 * 3.141592; # Calcurate the radian value. 

    show_description(page, font, 320, ypos - 60, "Rotating text");
    HPDF_Page_BeginText(page);
    HPDF_Page_SetTextMatrix(page, cos(rad1), sin(rad1), -sin(rad1), cos(rad1),
                330, ypos - 60);
    HPDF_Page_ShowText(page, "ABCabc123");
    HPDF_Page_EndText(page);

    # * Skewing text.
     
    show_description(page, font, 320, ypos - 120, "Skewing text");
    HPDF_Page_BeginText(page);

    angle1 = 10;
    angle2 = 20;
    rad1 = angle1 / 180 * 3.141592;
    rad2 = angle2 / 180 * 3.141592;

    HPDF_Page_SetTextMatrix(page, 1, tan(rad1), tan(rad2), 1, 320, ypos - 120);
    HPDF_Page_ShowText(page, "ABCabc123");
    HPDF_Page_EndText(page);

    # * scaling text(X direction)
     
    show_description(page, font, 320, ypos - 175, "Scaling text(X direction)");
    HPDF_Page_BeginText(page);
    HPDF_Page_SetTextMatrix(page, 1.5, 0, 0, 1, 320, ypos - 175);
    HPDF_Page_ShowText(page, "ABCabc12");
    HPDF_Page_EndText(page);

    # * scaling text(Y direction)
     
    show_description(page, font, 320, ypos - 250, "Scaling text(Y direction)");
    HPDF_Page_BeginText(page);
    HPDF_Page_SetTextMatrix(page, 1, 0, 0, 2, 320, ypos - 250);
    HPDF_Page_ShowText(page, "ABCabc123");
    HPDF_Page_EndText(page);

    # * char spacing, word spacing

    show_description(page, font, 60, 125, "char-spacing 0");
    show_description(page, font, 60, 105, "char-spacing 1.5");
    show_description(page, font, 60, 85, "char-spacing 1.5, word-spacing 2.5");

    HPDF_Page_SetFontAndSize(page, font, 16);
    HPDF_Page_SetRGBFill(page, 0.1, 0.3, 0.1);

    # char-spacing 0 
    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, 100, samp_text2);
    HPDF_Page_EndText(page);

    # char-spacing 1.5 
    HPDF_Page_SetCharSpace(page, 1.5);

    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, 80, samp_text2);
    HPDF_Page_EndText(page);

    # char-spacing 1.5, word-spacing 3.5 
    HPDF_Page_SetWordSpace(page, 2.5);

    HPDF_Page_BeginText(page);
    HPDF_Page_TextOut(page, 60, 60, samp_text2);
    HPDF_Page_EndText(page);

    # save the document to a file 
    HPDF_SaveToFile(pdf, fname);

    # clean up 
    HPDF_Free(pdf);

}

